<?php

namespace Drupal\vimeo_embed_field\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Json;
/**
 * Plugin implementation of the 'vimeo' formatter.
 *
 * @FieldFormatter(
 *   id = "vimeo_thumb",
 *   label = @Translation("Vimeo Thumbnail"),
 *   field_types = {
 *     "vimeo"
 *   }
 * )
 */
class VimeoThumbFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array( 
      'vimeo_thumb_size' => 'small', 
      'vimeo_target_blank' => 0, 
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $options = [ 
    'small' => '100px * 75px',
    'medium' => '200px * 150px',  
    'large' => '640px * 360px', 
    ]; 
    $elements['vimeo_thumb_size'] = array(
      '#type' => 'select', 
      '#options' => $options,
      '#title' => t('Vimeo Thumnail size'),
      '#default_value' => $this->getSetting('vimeo_thumb_size') ? $this->getSetting('vimeo_thumb_size') : "",
    );  
    $elements['vimeo_target_blank'] = array(
      '#type' => 'checkbox',
      '#title' => t('_blank'),
      '#default_value' => $this->getSetting('vimeo_target_blank'),
      '#description' => t('Opens the vimeo video in a new window or tab'),
    ); 
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $vimeo_thumb_size = $this->getSetting('vimeo_thumb_size');
    $summary[] = $this->t('Vimeo Thumbnail Size (@vimeo_thumb_size).', [
      '@vimeo_thumb_size' => $vimeo_thumb_size, 
    ]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];  
    foreach ($items as $delta => $item) { 
      $elements[$delta] = [ 
          '#theme' => 'vimeo_thumbnail',  
          '#vimeoInfo' => $this->getVimeoVideoInfo($item->vimeo_url),
          '#target' => $this->getSetting('vimeo_target_blank'),
      ]; 
    }    
    return $elements;
  } 

  /**
   * {@inheritdoc}
   */
  public function getVimeoVideoInfo($url = '') {  
    $thumbnail = []; 
    $vimeo_thumb_size = $this->getSetting('vimeo_thumb_size'); 
    $id = _get_vimeo_id_from_vimeo_url($url); 
    if (($id['status'] == 1) && (!empty($id))) {
      $vimeo_video_id = $id['video_id'];
      $thumbnail = $this->get_Vimeo_Video_Thumbnail_VideoId($vimeo_video_id, $vimeo_thumb_size);
    }
    return $thumbnail;
  } 

/**
 * Gets the thumbnail url for a vimeo video using the video id. This only works for public videos.
 *
 * @param string $id        The video id.
 * @param string $thumbType Thumbnail image size. supported sizes: small, medium (default) and large.
 *
 * @return string|bool
 */
function get_Vimeo_Video_Thumbnail_VideoId( $id = '', $thumbType = '' ) {
  $id = trim( $id );
  if ( $id == '' ) {
      return FALSE;
  }
  $url =  "https://vimeo.com/api/v2/video/$id.json";
  try {
    $response = \Drupal::httpClient()->get($url, array('headers' => array('Accept' => 'application/json')));
    $getBody =  $response->getBody();
    $data = json_decode ($getBody, TRUE);
    if ( (is_array( $data ) ) && (count( $data ) > 0) ) {  
      $videoInfo = $data[0];  
      $vimeoInfo[ 'title' ] = $videoInfo[ 'title' ];
      $vimeoInfo[ 'url' ] = $videoInfo[ 'url' ];
      switch ( $thumbType ) {
          case 'small':
              $vimeoInfo[ 'thumbnail' ] = $videoInfo[ 'thumbnail_small' ]; 
              return $vimeoInfo;
              break;
          case 'large':
              $vimeoInfo[ 'thumbnail' ] = $videoInfo[ 'thumbnail_large' ]; 
              return $vimeoInfo;
              break;
          case 'medium': 
              $vimeoInfo[ 'thumbnail' ] = $videoInfo[ 'thumbnail_medium' ]; 
              return $vimeoInfo;
          default:
              break;
      }
    }
  }
  catch (RequestException $e) {
      return FALSE;
  }
   
}
  

}
